@@ -426,8 +426,6 @@ void    cleanup_pre_jail(char *unused_name, char **unused_argv)
 					var_milt_eod_macros,
 					var_milt_unk_macros,
 					var_milt_macro_deflts);
-    if (*var_milt_head_checks)
-	cleanup_milter_header_checks_init();
 
     flush_init();
 }
@@ -449,7 +447,7 @@ void    cleanup_post_jail(char *unused_name, char **unused_argv)
      * really low limit, the difference is going to matter only when a queue
      * file has lots of recipients.
      */
-    if (var_message_limit > 0)
+    if (ENFORCING_SIZE_LIMIT(var_message_limit))
 	set_file_limit((off_t) var_message_limit);
 
     /*
